From 85285a2b0ab3c963aa3227e94ba3261bfdb035bb Mon Sep 17 00:00:00 2001
From: Alyssa Rosenzweig <alyssa@rosenzweig.io>
Date: Sat, 2 Oct 2021 15:55:17 -0400
Subject: [PATCH] PCI: apple: Bring up ports in parallel

Rather than the sequence:

	Bring up port #0
	Wait for port #0
	Bring up port #1
	Wait for port #1
	Bring up port #2
	Wait for port #3

Reorder as:

	Bring up port #0
	Bring up port #1
	Bring up port #2
	Wait for port #0
	Wait for port #1
	Wait for port #3

This allows ports #1 and #2 to come up in parallel with port #0,
reducing the amount of stalling in the driver at boot-time.

Signed-off-by: Alyssa Rosenzweig <alyssa@rosenzweig.io>
---
 drivers/pci/controller/pcie-apple.c | 15 +++++++++------
 1 file changed, 9 insertions(+), 6 deletions(-)

diff --git a/drivers/pci/controller/pcie-apple.c b/drivers/pci/controller/pcie-apple.c
index fc71ddd14b08..529a62437086 100644
--- a/drivers/pci/controller/pcie-apple.c
+++ b/drivers/pci/controller/pcie-apple.c
@@ -136,7 +136,6 @@ struct apple_pcie {
 	struct irq_domain	*domain;
 	unsigned long		*bitmap;
 	struct list_head	ports;
-	struct completion	event;
 	struct irq_fwspec	fwspec;
 	u32			nvecs;
 };
@@ -146,6 +145,7 @@ struct apple_pcie_port {
 	struct device_node	*np;
 	void __iomem		*base;
 	struct irq_domain	*domain;
+	struct completion	event;
 	struct list_head	entry;
 	DECLARE_BITMAP(		sid_map, MAX_RID2SID);
 	int			sid_map_sz;
@@ -412,7 +412,7 @@ static irqreturn_t apple_pcie_port_irq(int irq, void *data)
 	case PORT_INT_LINK_UP:
 		dev_info_ratelimited(port->pcie->dev, "Link up on %pOF\n",
 				     port->np);
-		complete_all(&port->pcie->event);
+		complete_all(&port->event);
 		break;
 	case PORT_INT_LINK_DOWN:
 		dev_info_ratelimited(port->pcie->dev, "Link down on %pOF\n",
@@ -571,16 +571,13 @@ static int apple_pcie_setup_port(struct apple_pcie *pcie,
 	port->sid_map_sz = i;
 
 	list_add_tail(&port->entry, &pcie->ports);
-	init_completion(&pcie->event);
+	init_completion(&port->event);
 
 	ret = apple_pcie_port_register_irqs(port);
 	WARN_ON(ret);
 
 	writel_relaxed(PORT_LTSSMCTL_START, port->base + PORT_LTSSMCTL);
 
-	if (!wait_for_completion_timeout(&pcie->event, HZ / 10))
-		dev_warn(pcie->dev, "%pOF link didn't come up\n", np);
-
 	return 0;
 }
 
@@ -756,6 +753,7 @@ static int apple_pcie_init(struct pci_config_window *cfg)
 	struct platform_device *platform = to_platform_device(dev);
 	struct device_node *of_port;
 	struct apple_pcie *pcie;
+	struct apple_pcie_port *port;
 	int ret;
 
 	pcie = devm_kzalloc(dev, sizeof(*pcie), GFP_KERNEL);
@@ -782,6 +780,11 @@ static int apple_pcie_init(struct pci_config_window *cfg)
 		}
 	}
 
+	list_for_each_entry(port, &pcie->ports, entry) {
+		if (!wait_for_completion_timeout(&port->event, HZ / 10))
+			dev_warn(pcie->dev, "%pOF link didn't come up\n", port);
+	}
+
 	return apple_msi_init(pcie);
 }
 
-- 
2.30.2

